@@ -145,6 +145,8 @@ class ProxyApplication:
     @staticmethod
     def get_url(request, remote, path=None):
         """Request remote URL getter"""
+        if not remote:
+            return None
         components = urlsplit(remote)
         if not path:
             path = request.url.path[1:]
@@ -155,16 +157,20 @@ class ProxyApplication:
                                        path='/{}'.format('/'.join(path[1:]))))
 
     @staticmethod
-    def get_headers(request, decode=False):
+    def get_headers(source, decode=False):
         """Request headers getter"""
-        headers = getattr(request.state, 'headers', None)
+        state = getattr(source, 'state', None)
+        headers = None
+        if state is not None:
+            headers = getattr(source.state, 'headers', None)
         if headers:
             headers = list(headers.items())
         else:
-            headers = request.headers.raw
+            headers = source.headers.raw
         if decode:
             return [
-                (key.decode(), value.decode())
+                (key.decode() if isinstance(key, bytes) else key,
+                 value.decode() if isinstance(value, bytes) else value)
                 for key, value in headers
                 if key not in ('host', b'host')
             ]
@@ -179,7 +185,8 @@ class ProxyApplication:
         """Request params getter"""
         if decode:
             return [
-                (key.decode(), value.decode())
+                (key.decode() if isinstance(key, bytes) else key,
+                 value.decode() if isinstance(value, bytes) else value)
                 for key, value in request.query_params.items()
             ]
         return [
